/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2025 Atmark-techno
 */

#ifndef __ASM_ARCH_IMX8ULP_M33_I2C_H
#define __ASM_ARCH_IMX8ULP_M33_I2C_H

#include <asm/arch/m33_proto.h>

/* constants from linux arch/arm64/boot/dts/freescale/imx8ulp-pinfunc-m33.h */
#define I2C_TYPE_LPI2C          0

/* helpers */

/* this only supports lpi2c */
static inline int m33_i2c_init(u8 id, u8 i2c_idx, u32 baudrate)
{
	int rc;
	u32 status;

	rc = m33_send(M33_I2C | (M33_I2C_INIT << 8) | (id << 16));
	if (rc)
		return rc;

	rc = m33_send(I2C_TYPE_LPI2C);
	if (rc)
		return rc;

	rc = m33_send(i2c_idx);
	if (rc)
		return rc;

	rc = m33_send(baudrate);
	if (rc)
		return rc;

	rc = m33_recv(10, &status);
	if (rc)
		return rc;

	return status;
}

static inline int m33_i2c_reset(u8 id)
{
	int rc;
	u32 status;

	rc = m33_send(M33_I2C | (M33_I2C_RESET << 8) | (id << 16));
	if (rc)
		return rc;

	rc = m33_recv(10, &status);
	if (rc)
		return rc;

	return status;
}

static inline int m33_i2c_read(u8 id, u32 addr, void *buf, u32 len)
{
	int rc;
	u32 status;

	rc = m33_send(M33_I2C | (M33_I2C_READ << 8) | (id << 16));
	if (rc)
		return rc;

	rc = m33_send(addr);
	if (rc)
		return rc;

	rc = m33_send(len);
	if (rc)
		return rc;

	rc = m33_recv(10, &status);
	if (rc)
		return rc;

	if (status)
		return status;

	return m33_recv_many(10, buf, len);
}

static inline int m33_i2c_write(u8 id, u32 addr, const void *buf, u32 len)
{
	int rc;
	u32 status;

	rc = m33_send(M33_I2C | (M33_I2C_WRITE << 8) | (id << 16));
	if (rc)
		return rc;

	rc = m33_send(addr);
	if (rc)
		return rc;

	rc = m33_send(len);
	if (rc)
		return rc;

	rc = m33_send_many(buf, len);
	if (rc)
		return rc;

	rc = m33_recv(10, &status);
	if (rc)
		return rc;

	return status;
}

static inline int m33_i2c_read_reg(u8 id, u32 addr, u8 subaddr, void *buf, u32 len)
{
	int rc;

	rc = m33_i2c_write(id, addr, &subaddr, 1);
	if (rc)
		return rc;

	return m33_i2c_read(id, addr, buf, len);
}

static inline int m33_i2c_write_reg(u8 id, u32 addr, u8 subaddr, const void *buf, u32 len)
{
	int rc;

	/* Note: original code would issue everything to uboot in a single write
	 * with both buffers concatenated, but this seems to work as is */
	rc = m33_i2c_write(id, addr, &subaddr, 1);
	if (rc)
		return rc;

	return m33_i2c_write(id, addr, buf, len);
}

#endif

